import { getLog } from './get-log';
import bindMethod from './bind-method';
import wrapEventFilter from './wrap-event-filter';
import wrapDispose from './wrap-dispose';

const getSecretsManager = pool => {
  return {
    deleteSecret: pool.deleteSecret,
    getSecret: pool.getSecret,
    setSecret: pool.setSecret
  };
};

const createAdapter = ({
  maybeThrowResourceError,
  validateEventFilter,
  loadEvents,
  importEventsStream,
  exportEventsStream,
  incrementalImport,
  getNextCursor,
  importSecretsStream,
  exportSecretsStream,
  init,
  drop,
  gatherSecretsFromEvents,
  getEventLoader
}, {
  dispose,
  loadEventsByCursor,
  loadEventsByTimestamp,
  ensureEventSubscriber,
  removeEventSubscriber,
  getEventSubscribers,
  getLatestEvent,
  saveEvent,
  initEvents,
  initSecrets,
  initFinal,
  dropEvents,
  dropSecrets,
  dropFinal,
  injectEvent,
  injectEvents,
  freeze,
  unfreeze,
  shapeEvent,
  loadSnapshot,
  saveSnapshot,
  dropSnapshot,
  beginIncrementalImport,
  commitIncrementalImport,
  rollbackIncrementalImport,
  pushIncrementalImport,
  getSecret,
  setSecret,
  deleteSecret,
  loadSecrets,
  injectSecret,
  replicateEvents,
  replicateSecrets,
  setReplicationPaused,
  setReplicationStatus,
  getReplicationState,
  resetReplication,
  setReplicationLock,
  getCursorUntilEventTypes,
  describe,
  establishTimeLimit,
  getEventLoaderNative,
  runtimeInfo,
  setReconnectionMode
}, options, configure) => {
  const log = getLog(`createAdapter`);
  const config = { ...options
  };
  let bucketSize = 100;
  const {
    snapshotBucketSize
  } = config;

  if (snapshotBucketSize !== undefined && Number.isSafeInteger(snapshotBucketSize) && snapshotBucketSize > 0) {
    bucketSize = snapshotBucketSize;
    log.debug(`snapshot bucket size explicitly set to ${bucketSize}`);
  } else {
    log.debug(`snapshot bucket size defaulted to ${bucketSize}`);
  }

  const emptyProps = {};
  const adapterPool = {
    disposed: false,
    validateEventFilter,
    isConnected: false,
    maybeThrowResourceError,
    bucketSize,
    getNextCursor: getNextCursor.bind(null),
    counters: new Map(),
    ...emptyProps
  };
  configure(adapterPool, config);
  const connectedProps = {
    injectEvent: bindMethod(adapterPool, injectEvent),
    injectEvents: bindMethod(adapterPool, injectEvents),
    injectSecret: bindMethod(adapterPool, injectSecret),
    loadEventsByCursor: bindMethod(adapterPool, loadEventsByCursor),
    loadEventsByTimestamp: bindMethod(adapterPool, loadEventsByTimestamp),
    deleteSecret: bindMethod(adapterPool, deleteSecret),
    getSecret: bindMethod(adapterPool, getSecret),
    setSecret: bindMethod(adapterPool, setSecret),
    initEvents: bindMethod(adapterPool, initEvents),
    initSecrets: bindMethod(adapterPool, initSecrets),
    initFinal: bindMethod(adapterPool, initFinal),
    dropEvents: bindMethod(adapterPool, dropEvents),
    dropSecrets: bindMethod(adapterPool, dropSecrets),
    dropFinal: bindMethod(adapterPool, dropFinal),
    shapeEvent,
    getEventLoaderNative: getEventLoaderNative !== undefined ? bindMethod(adapterPool, getEventLoaderNative) : getEventLoaderNative
  };
  Object.assign(adapterPool, connectedProps);
  const adapter = {
    loadEvents: bindMethod(adapterPool, wrapEventFilter(loadEvents)),
    importEvents: bindMethod(adapterPool, importEventsStream),
    exportEvents: bindMethod(adapterPool, exportEventsStream),
    getLatestEvent: bindMethod(adapterPool, getLatestEvent),
    saveEvent: bindMethod(adapterPool, saveEvent),
    init: bindMethod(adapterPool, init),
    drop: bindMethod(adapterPool, drop),
    dispose: wrapDispose(adapterPool, dispose),
    freeze: bindMethod(adapterPool, freeze),
    unfreeze: bindMethod(adapterPool, unfreeze),
    getNextCursor: getNextCursor.bind(null),
    getSecretsManager: bindMethod(adapterPool, getSecretsManager),
    loadSnapshot: bindMethod(adapterPool, loadSnapshot),
    saveSnapshot: bindMethod(adapterPool, saveSnapshot),
    dropSnapshot: bindMethod(adapterPool, dropSnapshot),
    pushIncrementalImport: bindMethod(adapterPool, pushIncrementalImport),
    beginIncrementalImport: bindMethod(adapterPool, beginIncrementalImport),
    commitIncrementalImport: bindMethod(adapterPool, commitIncrementalImport),
    rollbackIncrementalImport: bindMethod(adapterPool, rollbackIncrementalImport),
    incrementalImport: bindMethod(adapterPool, incrementalImport),
    loadSecrets: bindMethod(adapterPool, loadSecrets),
    importSecrets: bindMethod(adapterPool, importSecretsStream),
    exportSecrets: bindMethod(adapterPool, exportSecretsStream),
    ensureEventSubscriber: bindMethod(adapterPool, ensureEventSubscriber),
    removeEventSubscriber: bindMethod(adapterPool, removeEventSubscriber),
    getEventSubscribers: bindMethod(adapterPool, getEventSubscribers),
    gatherSecretsFromEvents: bindMethod(adapterPool, gatherSecretsFromEvents),
    replicateEvents: bindMethod(adapterPool, replicateEvents),
    replicateSecrets: bindMethod(adapterPool, replicateSecrets),
    setReplicationPaused: bindMethod(adapterPool, setReplicationPaused),
    setReplicationStatus: bindMethod(adapterPool, setReplicationStatus),
    getReplicationState: bindMethod(adapterPool, getReplicationState),
    resetReplication: bindMethod(adapterPool, resetReplication),
    setReplicationLock: bindMethod(adapterPool, setReplicationLock),
    getCursorUntilEventTypes: bindMethod(adapterPool, getCursorUntilEventTypes),
    describe: bindMethod(adapterPool, describe),
    establishTimeLimit: establishTimeLimit === undefined ? f => {
      return;
    } : establishTimeLimit.bind(null, adapterPool),
    getEventLoader: bindMethod(adapterPool, getEventLoader),
    runtimeInfo: runtimeInfo === undefined ? () => {
      return {
        connectionCount: 0,
        disposed: adapterPool.disposed
      };
    } : runtimeInfo.bind(null, adapterPool),
    setReconnectionMode: setReconnectionMode === undefined ? mode => {
      return;
    } : bindMethod(adapterPool, setReconnectionMode)
  };
  Object.assign(adapterPool, adapter);
  return Object.freeze(adapter);
};

export default createAdapter;
//# sourceMappingURL=create-adapter.js.map