"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _stream = require("stream");

var _constants = require("./constants");

var _getNextCursor = _interopRequireDefault(require("./get-next-cursor"));

var _frozenErrors = require("./frozen-errors");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const FLUSH_CHUNK_SIZE = 64 * 1024 * 1024;

async function startProcessEvents({
  pool,
  maintenanceMode,
  cursor,
  preferRegularEventLoader
}) {
  if (maintenanceMode === _constants.MAINTENANCE_MODE_AUTO) {
    try {
      await pool.freeze();
    } catch (error) {
      if (!_frozenErrors.AlreadyFrozenError.is(error)) {
        throw error;
      }
    }
  }

  return await pool.getEventLoader({
    cursor
  }, {
    preferRegular: preferRegularEventLoader
  });
}

async function endProcessEvents({
  pool,
  maintenanceMode
}, eventLoader) {
  try {
    if (maintenanceMode === _constants.MAINTENANCE_MODE_AUTO) {
      try {
        await pool.unfreeze();
      } catch (error) {
        if (!_frozenErrors.AlreadyUnfrozenError.is(error)) {
          throw error;
        }
      }
    }
  } finally {
    await eventLoader.close();
  }
}

async function* generator(context) {
  const {
    bufferSize
  } = context;
  let eventsByteSize = 0;
  const eventLoader = await startProcessEvents(context);

  while (true) {
    const {
      events
    } = await eventLoader.loadEvents(_constants.BATCH_SIZE);
    let currentTotalChunkSize = 0;
    let eventRecords = [];

    function* flushEvents() {
      if (eventRecords.length <= 0) {
        return;
      }

      yield Buffer.concat(eventRecords.map(record => record.buffer));
      context.cursor = (0, _getNextCursor.default)(context.cursor, eventRecords.map(record => record.event));
      eventRecords = [];
      currentTotalChunkSize = 0;
    }

    for (const event of events) {
      const buffer = Buffer.from(JSON.stringify(event) + '\n', 'utf8');
      const byteLength = buffer.byteLength;

      if (eventsByteSize + byteLength > bufferSize) {
        yield* flushEvents();
        context.isBufferOverflow = true;
        await endProcessEvents(context, eventLoader);
        return;
      }

      eventsByteSize += byteLength;

      if (currentTotalChunkSize + buffer.byteLength > FLUSH_CHUNK_SIZE) {
        yield* flushEvents();

        if (context.externalTimeout) {
          await endProcessEvents(context, eventLoader);
          return;
        }
      }

      eventRecords.push({
        buffer,
        event
      });
      currentTotalChunkSize += buffer.byteLength;
    }

    void (yield* flushEvents());

    if (events.length < _constants.BATCH_SIZE) {
      context.isEnd = true;
      await endProcessEvents(context, eventLoader);
      return;
    } else if (context.externalTimeout) {
      await endProcessEvents(context, eventLoader);
      return;
    }
  }
}

const exportEventsStream = (pool, {
  cursor = null,
  maintenanceMode = _constants.MAINTENANCE_MODE_AUTO,
  bufferSize = Number.POSITIVE_INFINITY,
  preferRegularEventLoader = false
} = {}) => {
  if (![_constants.MAINTENANCE_MODE_AUTO, _constants.MAINTENANCE_MODE_MANUAL].includes(maintenanceMode)) {
    throw new Error(`Wrong maintenance mode ${String(maintenanceMode)}`);
  }

  const context = {
    pool,
    maintenanceMode,
    cursor,
    bufferSize,
    isBufferOverflow: false,
    isEnd: false,
    externalTimeout: false,
    preferRegularEventLoader: preferRegularEventLoader
  };

  const stream = _stream.Readable.from(generator(context));

  stream.on('timeout', () => {
    context.externalTimeout = true;
  });
  Object.defineProperty(stream, 'cursor', {
    get() {
      return context.cursor;
    }

  });
  Object.defineProperty(stream, 'isBufferOverflow', {
    get() {
      return context.isBufferOverflow;
    }

  });
  Object.defineProperty(stream, 'isEnd', {
    get() {
      return context.isEnd;
    }

  });
  Object.defineProperty(stream, 'preferRegularEventLoader', {
    get() {
      return context.preferRegularEventLoader;
    }

  });
  return stream;
};

var _default = exportEventsStream;
exports.default = _default;
//# sourceMappingURL=export-events.js.map